#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "addnew.h"
#include <QDebug>
#include <QMenu>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    tableWidget = new TableViewX(this);
    setCentralWidget(tableWidget);

    setupGUI();
    loadSettings();
}

void MainWindow::messageReceived(const QString &text)
{
    AddNew *addNewDialog = new AddNew (this);
    addNewDialog->setURL(text);
    addNewDialog->exec();

    if ( addNewDialog->ac )
        addNew (addNewDialog->getUrl() , addNewDialog->getSaveTo());

    delete addNewDialog;
}

void MainWindow::setupGUI()
{    
    tableWidget->menu->addAction( ui->actionOpen_This_File);
    tableWidget->menu->addAction( ui->actionOpen_Folder);
    tableWidget->menu->addSeparator();
    tableWidget->menu->addAction(ui->actionCopy_link_address);

    connect (tableWidget , SIGNAL(DblClicked()) , SLOT(on_actionOpen_This_File_triggered()));

    QMenu *menu = new QMenu (this);

    QAction *activateAction = new QAction ( QIcon(":/img/quickopen.png") ,  tr("Activate") , menu );
    connect (activateAction , SIGNAL(triggered()) , SLOT(show()));
    menu->addAction(activateAction);
    menu->addAction(ui->actionAdd_New);

    menu->addSeparator();
    menu->addAction(ui->actionStart_Continue);
    menu->addAction(ui->actionPause);

    menu->addSeparator();
    menu->addAction(ui->actionQuit);

    trayIcon = new QSystemTrayIcon (this);
    trayIcon->setContextMenu(menu);
    trayIcon->setIcon (QIcon(":/img/quickopen.png"));
    trayIcon->show();

    connect(trayIcon,SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
            this,SLOT(showOrHide(QSystemTrayIcon::ActivationReason)));

}

void MainWindow::showOrHide(QSystemTrayIcon::ActivationReason reason)
{
    switch(reason){
    case QSystemTrayIcon::Trigger:
    case QSystemTrayIcon::DoubleClick:
        setVisible(isHidden());
        break;
    default:
        break;
    }
}


void MainWindow::saveSettings()
{
    QSettings settings;
    settings.setValue("Aria2c/Option" , aria2copts);

    settings.remove("URL");
    settings.beginGroup("URL");
    QMap<QString, QString>::const_iterator i = savedUrls.constBegin();
    while (i != savedUrls.constEnd())
    {
        settings.setValue(i.key().toUtf8().toHex(), i.value());
        ++i;
    }
    settings.endGroup();
}

void MainWindow::loadSettings()
{
    QSettings settings;
    aria2copts = settings.value("Aria2c/Option").toString();

    settings.beginGroup("URL");
    QStringList keys = settings.childKeys();

    foreach (QString key, keys)
    {
        addNew( QByteArray::fromHex(key.toAscii()) , settings.value( key ).toString());
    }
    settings.endGroup();
}

MainWindow::~MainWindow()
{
    saveSettings();

    QVector<Downloader*>::const_iterator it = mapping.begin();
    while ( it != mapping.end() )
    {
        (*it)->stop();
        it ++;
    }

    delete ui;
}

void MainWindow::errorString (int id, const QString &text)
{
    tableWidget->item(id , 1)->setText( text );
    infoUpdated();
}

void MainWindow::infoUpdated()
{
    tableWidget->resizeColumnsToContents();
}

void MainWindow::msgbox(const QString &text)
{
    QMessageBox::information(this , tr("Information") , text);
}

void MainWindow::addNew (const QString &url, const QString &saveTo)
{
    if ( savedUrls.contains(url) )
    {
        msgbox(tr("Error: URL already exists"));
        return;
    }

    dir.setPath(saveTo);
    if ( ! dir.exists() )
        dir.mkpath(dir.absolutePath());

    savedUrls[url] = saveTo;
    int cnt = tableWidget->rowCount();

    Downloader *downloader = new Downloader ( cnt , this);
    downloader->setFolder(saveTo);
    downloader->setOptions(aria2copts);

    mapping.push_back(downloader);

    tableWidget->setRowCount(cnt + 1);
    tableWidget->setItem(cnt , 0 , new QTableWidgetItem ( QIcon(":/img/download.png") , QString::number(cnt + 1))); // #
    tableWidget->setItem(cnt , 1 , new QTableWidgetItem (SPACES + QString("Download") + SPACES)); // STAT
    tableWidget->setCellWidget(cnt , 2 , downloader->name);       // NAME
    tableWidget->setCellWidget(cnt , 3 , downloader->spd); // SPD
    tableWidget->setCellWidget(cnt , 4 , downloader->eta);  // ETA

    QTableWidgetItem *folderItem = new QTableWidgetItem ( SPACES + saveTo + SPACES );
    folderItem->setData(Qt::AlignCenter , Qt::TextAlignmentRole);
    tableWidget->setItem(cnt , 5 , folderItem); // FOLDER

    tableWidget->setCellWidget(cnt , 6 , downloader->bar);   // PROGRESS
    tableWidget->setItem(cnt , 7 , new QTableWidgetItem (url)); // URL
    tableWidget->setCellWidget(cnt , 8 , downloader->size);   // SIZE

    connect (downloader , SIGNAL(infoUpdated()) , SLOT(infoUpdated()));
    connect (downloader , SIGNAL(finished(int)) , SLOT(finished(int)));
    connect (downloader , SIGNAL(error(int,QString)) , SLOT(errorString(int,QString)));

    downloader->begin(url);
}

void MainWindow::finished(int id)
{
    tableWidget->item(id , 0)->setIcon(QIcon(":/img/task-complete.png"));
    tableWidget->item(id , 1)->setText( tr("   Finished   ") );
    infoUpdated();

    trayIcon->showMessage(tr("Transfer completed") ,
                          QString("URL: %1\nSaved to: %2")
                          .arg( mapping.at(id)->getUrl() )
                          .arg( mapping.at(id)->getFolder() ) , QSystemTrayIcon::Information , 5000);
}

void MainWindow::on_actionAdd_New_triggered()
{
    show();
    AddNew *addNewDialog = new AddNew (this);
    addNewDialog->exec();

    if ( addNewDialog->ac )
        addNew (addNewDialog->getUrl() , addNewDialog->getSaveTo());
}

void MainWindow::on_actionPause_triggered()
{
    int idx = tableWidget->currentIndex().row();
    if ( idx != -1 )
    {
        mapping.at( idx )->stop();
        tableWidget->itemAt(0 , idx)->setIcon(QIcon(":/img/user-away-extended.png"));
    }
}

void MainWindow::on_actionDelete_triggered()
{
    int idx = tableWidget->currentIndex().row();
    if ( idx != -1 )
    {
        mapping.at(idx)->stop();

        const QString & fName = mapping.at(idx)->getFolder() + "/" + mapping.at(idx)->getName().trimmed();
        fp.setFileName(fName);

        if ( fp.exists() && question( tr("Also remove %1 ? ").arg(fName) ) )
        {
            fp.remove();
            fp.setFileName(fName + ".aria2");

            if ( fp.exists() )
                fp.remove();
        }

        tableWidget->removeRow(idx);
        savedUrls.remove(mapping.at(idx)->getUrl());

        mapping.remove( idx );
    }
}

void MainWindow::on_actionStart_Continue_triggered()
{
    int idx = tableWidget->currentIndex().row();
    if ( idx != -1 )
    {
        mapping.at( idx )->start();
        tableWidget->itemAt(0 , idx)->setIcon(QIcon(":/img/download.png"));
    }
}

void MainWindow::on_actionAbout_triggered()
{
    QMessageBox::about(this , QApplication::applicationName() , tr("A simple UI for aria2c , written by <b>Aaron Lewis</b>"));
}

void MainWindow::on_actionOpen_Folder_triggered()
{
    int idx = tableWidget->currentIndex().row();
    if ( idx != -1 )
    {
        QProcess::startDetached("xdg-open " + mapping.at( idx )->getFolder());
    }
}

bool MainWindow::question(const QString &text)
{
    return QMessageBox::question(this , tr("Question") , text , QMessageBox::Yes , QMessageBox::No) == QMessageBox::Yes;
}

void MainWindow::on_actionQuit_triggered()
{
    if ( question (tr("Close window and shutdown all tasks ? ")) )
        close();
}

void MainWindow::on_actionConfigure_Aria2c_triggered()
{
    QMessageBox::warning(this , tr("Be careful !") ,
                         tr("Override default options by supplying new one here , seperate each one with space") , QMessageBox::Ok);
    const QString & text = QInputDialog::getText(this , tr("Configure aria2c command , careful ! ") ,
                                                 tr("e.g:  --file-alloc=none -c ") , QLineEdit::Normal , aria2copts);

    if ( ! text.isEmpty() )
    {
        aria2copts = text;
        for ( int i = 0 ; i < mapping.size() ; ++ i )
        {
            mapping[i]->setOptions(aria2copts);
        }
    }
}

void MainWindow::on_actionOpen_This_File_triggered()
{
    int idx = tableWidget->currentIndex().row();
    if ( idx != -1 )
    {
        const QString & fName = mapping.at(idx)->getFolder() + "/" + mapping.at(idx)->getName().trimmed();
        if ( ! fName.isEmpty() )
            QDesktopServices::openUrl( QUrl (fName) );
    }
}

void MainWindow::on_actionCopy_link_address_triggered()
{
    int id = tableWidget->currentIndex().row();
    if ( id != -1 )
    {
        QClipboard *clip = QApplication::clipboard();
        clip->setText(mapping.at(id)->getUrl());
    }
}
